#ifndef __CHM_IGC_DLL_H__
#define __CHM_IGC_DLL_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2009 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMIGCdll
//
// Description:
//
// CHMIGCdll class.
//
// Author: Steven Dodd
// Date:   Jan 19, 2009
// Revision: $Revision: 1.8 $
//
// Last Edit Date: $Date: 2009-02-26 17:37:57 $
// Source: $Source: /home/cvs/cvsroot/CHM/Attic/CHMIGCdll.h,v $
//---------------------------------------------------------------------------
/// @file


#include <CHM/CHMminimumInclude.h>

/** Disconnect reason flags.
 *  Provided to onDisconnect callbacks.
 **/
enum CHMIGCdisconnectReasonFlags
{
   CHMIGC_DISCONNECT_NONE        =  0,       ///< Default value - not a valid set of flags
   CHMIGC_DISCONNECT_BY_PLUGIN   =  1 << 0,  ///< Plugin called disconnect() or IGCmessageProcessorDestroy()
   CHMIGC_DISCONNECT_BY_IGUANA   =  1 << 1,  ///< Iguana closed the connection because the channel is stopping
   CHMIGC_DISCONNECT_BY_ERROR    =  1 << 2,  ///< Connection closed due to error
   CHMIGC_DISCONNECT_ALL         =  CHMIGC_DISCONNECT_BY_PLUGIN | CHMIGC_DISCONNECT_BY_IGUANA | CHMIGC_DISCONNECT_BY_ERROR /** Mask of all flags */
};

/** Log message types.
 *  Passed to CHMIGCmessageReceivedResponse::addLogMessage and CHMIGCmessageRequestResponse::addLogMessage.
 *  These correspond to the log message types supported by the Iguana logging system itself.
 **/
enum CHMIGClogMessageType
{
   CHMIGC_LOG_MESSAGE_NIL     =  0, ///< Default value - not a valid log message type
   CHMIGC_LOG_MESSAGE_ERROR   =  1, ///< Error-level log message
   CHMIGC_LOG_MESSAGE_INFO    =  2, ///< Info-level log message
   CHMIGC_LOG_MESSAGE_DEBUG   =  3  ///< Debug-level log message
};

// IGC callback function definitions
typedef void (CHM_STD_CALL *TCHMIGCmessageReceivedCallback)( void* /*StubHandle*/, const char* /*Message*/, void* /*MessageReceivedInternalHandle*/);
typedef void (CHM_STD_CALL *TCHMIGCmessageResubmitPreviewCallback)( void* /*StubHandle*/, const char* /*Message*/, void* /*MessageReceivedInternalHandle*/);
typedef void (CHM_STD_CALL *TCHMIGCmessageRequestCallback)( void* /*StubHandle*/, void* /*MessageRequestInternalHandle*/);
typedef void (CHM_STD_CALL *TCHMIGCmessageRequestCompletedCallback)( void* /*StubHandle*/, const char* /*Message Id*/, CHMboolean /*Processed*/);
typedef void (CHM_STD_CALL *TCHMIGCshutdownRequestCallback)( void* /*StubHandle*/ );
typedef void (CHM_STD_CALL *TCHMIGCdisconnectCallback)( void* /*StubHandle*/, CHMuint32 /*ReasonFlags*/, const char * /*ReasonDescription*/);

class CHMIGCdllPrivate;

/** Manages interaction with the IGC dynamic library.
 * Plugins need only limited interaction with this class.  Generally, a plugin will (1) create an instance 
 * of this class, (2) call loadIgcDll() to load the IGC dynamic library, and then (3) pass the instance of 
 * this class to the constructor of a CHMIGCsource or CHMIGCdestination object.
 **/
class CHMIGCdll
{
public:
   /** Default constructor. */
   CHMIGCdll();
   
   /** Destructor.  Non-virtual: do not inherit from this class. */
   ~CHMIGCdll();

   /** Platform-specific default name of the IGC dynamic library.
    * On Windows, "IGC.dll".  On Posix platforms, "libIGC.so", "libIGC.dylib", ...
    **/
   static const char * defaultIgcDllName();

   /** Loads the IGC dynamic library using the path provided.
    * If only the name of the dynamic library is provided, ensure that the system's dynamic loader
    * search path environment variable includes the directory within which the named dynamic library resides.
    * This environment variable is named PATH on Windows, and LD_LIBRARY_PATH on Posix platforms.
    **/
   void loadIgcDll(const char* Path = defaultIgcDllName());

   /** Returns the path to the loaded IGC dynamic library.
    * On Windows, this will be the complete path to the loaded IGC dynamic library.
    * On Posix, this will simply be the path provided to loadIgcDll().
    **/
   CHMstring completeDllPath() const;

   //
   // Internal methods used by other CHMIGC classes, e.g. CHMIGCsource, CHMIGCdestination, ...
   //
   void* messageProcessorCreate(void* StubHandle);
   void messageProcessorDestroy(void* ProcessorHandle);
   void messageProcessorSetDebugFileName(void* ProcessorHandle, const char* FileName);
   
   void messageProcessorConnect(void* ProcessorHandle, const char* Host, CHMuint32 Port, const char* pClientId);
   void* messageProcessorConnectNoThrow(void* ProcessorHandle, const char* Host, CHMuint32 Port, const char* pClientId);
   void messageProcessorDisconnect(void* ProcessorHandle);

   void connectForMessageReceived(void* pProcessorHandle, TCHMIGCmessageReceivedCallback);
   void connectForMessageResubmitPreview(void* pProcessorHandle, TCHMIGCmessageResubmitPreviewCallback);
   void connectForMessageRequest(void* pProcessorHandle, TCHMIGCmessageRequestCallback);
   void connectForMessageRequestCompleted(void* pProcessorHandle, TCHMIGCmessageRequestCompletedCallback);
   void connectForShutdownRequest(void* pProcessorHandle, TCHMIGCshutdownRequestCallback);
   void connectForDisconnect(void* pProcessorHandle, TCHMIGCdisconnectCallback);

   void* exceptionCreate(void* pStubHandle);
   void exceptionDestroy(void* pExceptionHandle);
   const char* exceptionGetDescription(void* pExceptionHandle);
   void exceptionSetDescription(void* pExceptionHandle, const char* pDesc);
   void* exceptionCopy(void* pStubHandle, void* pOrigException);   //returns a copy of the input handle

   // CHMIGCmessageReceivedResponse
   void messageReceivedResponseSetProcessed( void* pResponseHandle, CHMboolean NewValue );
   CHMboolean messageReceivedGetResponseProcessed( void* pResponseHandle );

   void messageReceivedResponseSetLogMessage( void* pResponseHandle, const char * );
   const char * messageReceivedResponseGetLogMessage( void* pResponseHandle );

   void messageReceivedResponseSetStopChannel( void* pResponseHandle, CHMboolean );
   CHMboolean messageReceivedResponseGetStopChannel( void* pResponseHandle );

   void messageReceivedResponseSetStopChannelErrorFlag( void* pResponseHandle, CHMboolean );
   CHMboolean messageReceivedResponseGetStopChannelErrorFlag( void* pResponseHandle );

   void messageReceivedResponseSetStopChannelReason( void* pResponseHandle, const char* );
   const char* messageReceivedResponseGetStopChannelReason( void* pResponseHandle );

   void messageReceivedResponseSetResendMessage( void* pResponseHandle, CHMboolean );
   CHMboolean messageReceivedResponseGetResendMessage( void* pResponseHandle );

   void messageReceivedResponseSetResendMessageDelay( void* pResponseHandle, CHMuint32 );
   CHMuint32 messageReceivedResponseGetResendMessageDelay( void* pResponseHandle );

   void messageReceivedResponseAddLogMessage( void* pResponseHandle, CHMuint32, const char* );

   const char* messageReceivedResponseGetLog( void* pResponseHandle, const char* pKey);
   void messageReceivedResponseAddLog( void* pResponseHandle, const char* pKey, const char* pValue);
   void messageReceivedResponseRemoveLog( void* pResponseHandle, const char* pKey);

   // CHMIGCmessageRequestResponse
   void messageRequestResponseSetMessageId( void* pResponseHandle, const char* pMessageId );
   const char* messageRequestResponseGetMessageId( void* pResponseHandle );
   
   void messageRequestResponseSetMessage( void* pResponseHandle, const char* pMessage);
   const char* messageRequestResponseGetMessage( void* pResponseHandle );

   CHMboolean messageRequestResponseGetIsError( void* pResponseHandle );
   void messageRequestResponseSetIsError( void* pResponseHandle, CHMboolean );

   CHMboolean messageRequestResponseGetIsLogMessage( void* pResponseHandle );
   void messageRequestResponseSetIsLogMessage( void* pResponseHandle, CHMboolean );

   CHMboolean messageRequestResponseGetStopChannel( void* pResponseHandle );
   void messageRequestResponseSetStopChannel( void* pResponseHandle, CHMboolean );

   CHMboolean messageRequestResponseGetStopChannelErrorFlag( void* pResponseHandle );
   void messageRequestResponseSetStopChannelErrorFlag( void* pResponseHandle, CHMboolean );

   const char* messageRequestResponseGetStopChannelReason( void* pResponseHandle );
   void messageRequestResponseSetStopChannelReason( void* pResponseHandle, const char* );

   void messageRequestResponseAddLogMessage( void* pResponseHandle, CHMuint32, const char* );

   const char* messageRequestResponseGetLog( void* pResponseHandle, const char* pKey);
   void messageRequestResponseAddLog( void* pResponseHandle, const char* pKey, const char* pValue);
   void messageRequestResponseRemoveLog( void* pResponseHandle, const char* pKey);

private:
   CHMIGCdllPrivate * pMember;
   CHMIGCdll(const CHMIGCdll& Orig); // not allowed
   CHMIGCdll& operator=(const CHMIGCdll& Orig); // not allowed
};

#endif // end of defensive include
