#ifndef __CHM_IGC_SOURCE_H__
#define __CHM_IGC_SOURCE_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2008 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMIGCsource
//
// Author: Steven Dodd
// Date:   Jan 20, 2009
// Revision: $Revision: 1.8 $
//
// Last Edit Date: $Date: 2009-02-19 03:35:02 $
// Source: $Source: /home/cvs/cvsroot/CHM/Attic/CHMIGCsource.h,v $
//---------------------------------------------------------------------------

#include <CHM/CHMminimumInclude.h>
#include <CHM/CHMstring.h>
#include <CHM/CHMerrorClass.h>
#include <CHM/CHMIGCdll.h>
#include <CHM/CHMIGCmessageRequestResponse.h>

/** Abstract base class for Iguana source plugins.
 * To create an Iguana source plugin, inherit from this class and provide implementations
 * for the pure-virtual event handler methods.
 **/
class CHMIGCsource
{
public:
   /** Default constructor. */
   CHMIGCsource(CHMIGCdll& Dll);

   /** Destructor. */
   virtual ~CHMIGCsource();

   /** Connects to an Iguana channel's source component.
    * \c Hostname should be set to "localhost" if the plugin is running on the same machine as Iguana,
    * or the hostname or IP address of the remote machine running Iguana.
    * \c Port should be set to the port Iguana's RPC server has been configured to listen on.
    * \c ChannelName should be set to the name of the desired channel.
    * Note that if Iguana starts the plugin executable, the channel may be configured to automatically 
    * pass appropriate values for these parameters using the following macros in the executable
    * parameters field: $hostname$ $port$ $channel$.
    **/
   void connectSource(const CHMstring& Hostname, CHMuint16 Port, const CHMstring& ChannelName);
  
   /** Disconnects from the Iguana channel's source component. */
   void disconnectSource();

   /** Sets a file name for debugging internal run-time information.  The file path supplied must be valid. 
    * The log file has a maximum size of 1 MB. When the file reaches 1 MB in size it is moved to FileName.old and 
    * a new file is created. Setting the file name to an empty string will disable debug logging.
    * This function is not thread safe, and should only be called before calling connect on the instance.  
    * This should be only used for support purposes to provide debug information to the development team at iNTERFACEWARE.
    **/
   void setDebugFileName(const CHMstring& FileName);

   /** This event is dispatched when Iguana is polling for a generated message. 
    * Note that exceptions can never be thrown out of this handler. Any exceptions thrown will result in the plugin
    * aborting. It is recommended that logging or the handling of exceptions is performed in the handler.
    **/
   virtual void onMessageRequest(CHMIGCmessageRequestResponse& RequestResponse)=0;

   /** Once a message request is completed, this event is received. This indicates to the client that any transactions 
    * started by the request of the message can be committed.  MessageSucceeded indicates if the message was logged 
    * as a success or error.  The received Plugin Message ID indicates that the message was processed.  If the message 
    * was processed in error by Iguana, this event is not received.  As a result, when the next message request is received, 
    * the previous message generation operations should be rolled back.
    * Note that exceptions can never be thrown out of this handler. Any exceptions thrown will result in the plugin
    * aborting. It is recommended that logging or the handling of exceptions is performed in the handler.
    **/
   virtual void onMessageRequestCompleted(const CHMstring& PluginMessageId, CHMboolean Processed)=0;

	/** This event is dispatched when the connection with Iguana is closed. The general reason for the disconnection is 
    * provided in the \c ReasonFlags, while additional details may be provided in the \c ReasonDescription.
    * Note that exceptions can never be thrown out of this handler. Any exceptions thrown will result in the plugin
    * aborting. It is recommended that logging or the handling of exceptions is performed in the handler.
    **/
   virtual void onDisconnectSource(CHMIGCdisconnectReasonFlags ReasonFlags, const CHMstring& ReasonDescription)=0;

	/** This method is deprecated in Iguana 4.0.  It has been superseded by onDisconnectSource.
    * This event is dispatched when the connection with Iguana is closed for any reason other than the plugin calling 
    * disconnect (or destroying the CHMIGCsource object).  Typically, this event is received when the Iguana channel 
    * is stopping, however, it may also be received upon disconnection due to network failures or other error conditions.
    * Note that exceptions can never be thrown out of this handler. Any exceptions thrown will result in the plugin
    * aborting. It is recommended that logging or the handling of exceptions is performed in the handler.
    **/
   virtual void onShutdownSource() {}
   
   CHMIGCdll& dll(){return Dll;}
         
private:
   CHMIGCdll& Dll;
   void* Handle;

   CHMIGCsource(const CHMIGCsource& Orig); // not allowed
   CHMIGCsource& operator=(const CHMIGCsource& Orig); // not allowed
};

#endif // end of defensive include
